@@ -1,9 +1,43 @@
+;*---------------------------------------------------------------------*/
+;*    Copyright (c) 1996 by Manuel Serrano. All rights reserved.       */
+;*                                                                     */
+;*                                     ,--^,                           */
+;*                               _ ___/ /|/                            */
+;*                           ,;'( )__, ) '                             */
+;*                          ;;  //   L__.                              */
+;*                          '   \    /  '                              */
+;*                               ^   ^                                 */
+;*                                                                     */
+;*                                                                     */
+;*    This program is distributed in the hope that it will be useful.  */
+;*    Use and copying of this software and preparation of derivative   */
+;*    works based upon this software are permitted, so long as the     */
+;*    following conditions are met:                                    */
+;*           o credit to the authors is acknowledged following         */
+;*             current academic behaviour                              */
+;*           o no fees or compensation are charged for use, copies,    */
+;*             or access to this software                              */
+;*           o this copyright notice is included intact.               */
+;*      This software is made available AS IS, and no warranty is made */
+;*      about the software or its performance.                         */
+;*                                                                     */
+;*      Bug descriptions, use reports, comments or suggestions are     */
+;*      welcome Send them to                                           */
+;*        <Manuel.Serrano@inria.fr>                                    */
+;*        Manuel Serrano                                               */
+;*        INRIA -- Rocquencourt                                        */
+;*        Domaine de Voluceau, BP 105                                  */
+;*        78153 Le Chesnay Cedex                                       */
+;*        France                                                       */
+;*---------------------------------------------------------------------*/
+
+
 ;*---------------------------------------------------------------------*/
 ;*    serrano/prgm/project/bigloo/examples/Pp/pp.scm ...               */
 ;*                                                                     */
 ;*    Author      :  Marc Feeley (c)                                   */
 ;*    Creation    :  Thu Jan  7 13:02:43 1993                          */
-;*    Last change :  Mon May 10 08:31:32 1993  (serrano)               */
+;*    Last change :  Sat Nov 12 16:06:23 1994 (serrano)                */
 ;*                                                                     */
 ;*    Un petit pretty-printer pour Scheme.                             */
 ;*---------------------------------------------------------------------*/
@@ -11,7 +45,10 @@
 ;*---------------------------------------------------------------------*/
 ;*    Le module                                                        */
 ;*---------------------------------------------------------------------*/
-(module pp (main main))
+(module pp
+   (main main)
+   (export *case* *ignore-comment*)
+   (import (pp-reader "reader.scm")))
 
 ;*---------------------------------------------------------------------*/
 ;*    *width* ...                                                      */
@@ -24,6 +61,16 @@
 (define *output* (current-output-port))
 (define *output-name* '())
 
+;*---------------------------------------------------------------------*/
+;*    the *case* ...                                                   */
+;*---------------------------------------------------------------------*/
+(define *case* 'respect)
+
+;*---------------------------------------------------------------------*/
+;*    Les commentaires                                                 */
+;*---------------------------------------------------------------------*/
+(define *ignore-comment* #f)
+
 ;*---------------------------------------------------------------------*/
 ;*    main ...                                                         */
 ;*---------------------------------------------------------------------*/
@@ -52,14 +99,8 @@
 	 ((null? args)
 	  '())
 	 ((string=? (car args) "-help")
-	  (print "usage: pp [-o output-file] [-wWIDTH] [file1] ... [fileN]")
+	  (print "usage: pp [-comment] [-o output-file] [-upper] [-lower] [-wWIDTH] [file1] ... [fileN]")
 	  (exit -1))
-	 ((null? (cdr args))
-	  (if (not (file-exists? (car args)))
-	      (begin
-		 (set! *output-name* (car args))
-		 '())
-	      (list (car args))))
 	 ((and (> (string-length (car args)) 2)
 	       (string=? (substring (car args) 0 2) "-w"))
 	  (set! *width* (string->integer
@@ -73,6 +114,21 @@
 	      (begin
 		 (set! *output-name* (cadr args))
 		 (loop (cddr args)))))
+	 ((string=? (car args) "-upper")
+	  (set! *case* 'upper)
+	  (loop (cdr args)))
+	 ((string=? (car args) "-lower")
+	  (set! *case* 'lower)
+	  (loop (cdr args)))
+	 ((string=? (car args) "-comment")
+	  (set! *ignore-comment* #t)
+	  (loop (cdr args)))
+	 ((null? (cdr args))
+	  (if (not (file-exists? (car args)))
+	      (begin
+		 (set! *output-name* (car args))
+		 '())
+	      (list (car args))))
 	 (else
 	  (cons (car args) (loop (cdr args)))))))
 
@@ -81,7 +137,7 @@
 ;*    pp-console ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (pp-console)
-   (let loop ((exp (read (current-input-port))))
+   (let loop ((exp (pp-read (current-input-port))))
       (if (eof-object? exp)
 	  #T
 	  (begin
@@ -90,7 +146,7 @@
 			    *width*
 			    (lambda (s) (display s *output*) #t))
 	     (flush-output-port *output*)
-	     (loop (read (current-input-port)))))))
+	     (loop (pp-read (current-input-port)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    pp-files ...                                                     */
@@ -109,7 +165,7 @@
 			      (print (car list))
 			      (print "::::::::::::::")))
 		       (let ((port (open-input-file (car list))))
-			  (let loop ((exp (read port)))
+			  (let loop ((exp (pp-read port)))
 			     (if (eof-object? exp)
 				 (close-input-port port)
 				 (begin
@@ -119,9 +175,20 @@
 						   (lambda (s)
 						      (display s *output*)
 						      #t))
-				    (loop (read port))))))))
+				    (loop (pp-read port))))))))
 		(loop (cdr list)))))))
        
+;; les prefix des vecteurs
+(define (vector-prefix obj)
+   (let ((tag (vector-tag obj)))
+      (if (=fx tag 0)
+	  "#"
+	  (if (>=fx tag 100)
+	      (string-append "#" (number->string tag))
+	      (if (>=fx tag 10)
+		  (string-append "#0" (number->string tag))
+		  (string-append "#00" (number->string tag)))))))
+
 ; 'generic-write' is a procedure that transforms a Scheme data value (or
 ; Scheme program expression) into its textual representation.  The interface
 ; to the procedure is sufficiently general to easily implement other useful
@@ -187,32 +254,60 @@
                      (else      (out ")" (wr l (out " . " col)))))))
         (out "()" col)))
 
-    (cond ((pair? obj)        (wr-expr obj col))
+    (cond ((match-case obj
+	      ((COMMENT (? integer?) (? string?))
+	       #t)
+	      (else
+	       #f))
+	   (let ((add (- *width* (string-length (caddr obj)) 3)))
+	      (if (<= add 0)
+		  (out (caddr obj) col)
+		  (out (string-append (caddr obj) (make-string add #\space))
+		       col))))
+	  ((pair? obj)        (wr-expr obj col))
           ((null? obj)        (wr-lst obj col))
-          ((vector? obj)      (wr-lst (vector->list obj) (out "#" col)))
+          ((vector? obj)      (wr-lst (vector->list obj)
+				      (out (vector-prefix obj) col)))
           ((boolean? obj)     (out (if obj "#t" "#f") col))
           ((number? obj)      (out (number->string obj) col))
-          ((symbol? obj)      (out (symbol->string obj) col))
-          ((procedure? obj)   (out "#[procedure]" col))
-          ((string? obj)      (if display?
-                                (out obj col)
-                                (let loop ((i 0) (j 0) (col (out "\"" col)))
-                                  (if (and col (< j (string-length obj)))
-                                    (let ((c (string-ref obj j)))
-				       (loop i (+ j 1) col))
-                                    (out "\""
-                                         (out (substring obj i j) col))))))
+          ((symbol? obj)      (case *case*
+				 ((respect)
+				  (out (symbol->string obj) col))
+				 ((upper)
+				  (out (string-upcase (symbol->string obj))
+				       col))
+				 (else
+				  (out (string-downcase (symbol->string obj))
+				       col))))
+          ((procedure? obj)   (out "#<procedure>" col))
+          ((string? obj)      (let ((obj (string-for-read obj)))
+				 (if display?
+				     (out obj col)
+				     (let loop ((i 0)
+						(j 0) 
+						(col (out "#\"" col)))
+					(if (and col (< j (string-length obj)))
+					    (let ((c (string-ref obj j)))
+					       (loop i (+ j 1) col))
+					    (out "\""
+						 (out (substring obj i j)
+						      col)))))))
           ((char? obj)        (if display?
                                 (out (make-string 1 obj) col)
                                 (out (case obj
                                        ((#\space)   "space")
                                        ((#\newline) "newline")
+				       ((#\return)  "return")
+				       ((#\tab)     "tab")
                                        (else        (make-string 1 obj)))
                                      (out "#\\" col))))
-          ((input-port? obj)  (out "#[input-port]" col))
-          ((output-port? obj) (out "#[output-port]" col))
-          ((eof-object? obj)  (out "#[eof-object]" col))
-          (else               (out "#[unknown]" col))))
+          ((input-port? obj)  (out "#<input-port>" col))
+          ((output-port? obj) (out "#<output-port>" col))
+          ((eof-object? obj)  (out "#<eof-object>" col))
+          (else               (out (let ((p (open-output-string)))
+				      (write obj p)
+				      (close-output-port p))
+				   col))))
 
   (define (pp obj col)
 
@@ -242,7 +337,9 @@
             (out (reverse-string-append result) col)
             (if (pair? obj)
               (pp-pair obj col extra)
-              (pp-list (vector->list obj) (out "#" col) extra pp-expr))))
+              (pp-list (vector->list obj)
+		       (out (vector-prefix obj) col)
+		       extra pp-expr))))
         (wr obj col)))
 
     (define (pp-expr expr col extra)
@@ -277,6 +374,14 @@
       (let ((col (out "(" col)))
         (pp-down l col col extra pp-item)))
 
+    ; (item1 item2 item3
+    ;    item4)
+    (define (pp-defun-form expr col extra pp-item)
+       (let* ((col  (out "({}" col))
+	      (col2 (wr (car expr) col))
+	      (col3 (wr (cadr expr) col2)))
+	  (pp-down (cddr expr) col3 (+ col3 1) extra pp-item)))
+       
     (define (pp-down l col1 col2 extra pp-item)
       (let loop ((l l) (col col1))
         (and col
@@ -328,9 +433,40 @@
     (define (pp-expr-list l col extra)
       (pp-list l col extra pp-expr))
 
+    (define (pp-expr-defun l col extra)
+      (pp-defun-form l col extra pp-expr))
+    
+    (define (pp-DEFiNE expr col extra)
+      (pp-general expr col extra #f pp-expr-list #f pp-expr)
+      (out #"\n" 0))
+
+    (define (pp-MODULE expr col extra)
+      (pp-general expr col extra #f pp-expr #f pp-expr-list)
+      (out #"\n" 0))
+
+    (define (pp-DEFUN expr col extra)
+      (pp-general expr col extra #t pp-expr-defun #f pp-expr)
+      (out #"\n" 0))
+      
     (define (pp-LAMBDA expr col extra)
       (pp-general expr col extra #f pp-expr-list #f pp-expr))
 
+    (define (pp-COMMENT expr col extra)
+       (match-case expr
+	  ((COMMENT (and (? integer?) ?column) (and (? string?) ?string))
+	   (let ((add (- *width* (string-length string) 3)))
+	      (if (=fx column 0)
+		  (if (> add 0)
+		      (out (string-append string (make-string add #\space))
+			   0)
+		      (out string 0))
+		  (if (> add 0)
+		      (out (string-append string (make-string add #\space))
+			   col)
+		      (out string col)))))
+	  (else
+	   (pp-general expr col extra #f pp-expr #f pp-expr))))
+       
     (define (pp-IF expr col extra)
       (pp-general expr col extra #f pp-expr #f pp-expr))
 
@@ -363,16 +499,22 @@
     (define max-expr-width 50)
 
     (define (style head)
-      (case head
-        ((LAMBDA LET* LETREC DEFINE) pp-LAMBDA)
-        ((IF SET!)                   pp-IF)
-        ((COND)                      pp-COND)
-        ((CASE)                      pp-CASE)
-        ((AND OR)                    pp-AND)
-        ((LET)                       pp-LET)
-        ((BEGIN)                     pp-BEGIN)
-        ((DO)                        pp-DO)
-        (else                        #f)))
+      (case (if (eq? *case* 'respect)
+		(string->symbol (string-upcase (symbol->string head)))
+		head)
+        ((LAMBDA LET* LETREC)   pp-LAMBDA)
+	((DEFINE DEFINE-INLINE) pp-DEFINE)
+	((MODULE)               pp-MODULE) 
+	((DEFUN DE)             pp-DEFUN)
+        ((IF SET!)              pp-IF)
+        ((COND)                 pp-COND)
+        ((CASE)                 pp-CASE)
+        ((AND OR)               pp-AND)
+        ((LET)                  pp-LET)
+        ((BEGIN)                pp-BEGIN)
+        ((DO)                   pp-DO)
+	((COMMENT)              pp-COMMENT)
+        (else                   #f)))
 
     (pr obj col 0 pp-expr))
 
@@ -393,7 +535,7 @@
           (if (< j len)
             (begin
               (string-set! result k (string-ref str j))
-              (loop (+ j 1) (+ k 1)))
+              (loop (+ j 1) (+ k 1))) 
             result)))
       (make-string i)))
 
