@@ -1,5 +1,5 @@
 /*---------------------------------------------------------------------*/
-/*    Copyright (c) 1993 by Manuel Serrano. All rights reserved.       */
+/*    Copyright (c) 1996 by Manuel Serrano. All rights reserved.       */
 /*                                                                     */
 /*                                     ,--^,                           */
 /*                               _ ___/ /|/                            */
@@ -33,28 +33,50 @@
 
 
 /*---------------------------------------------------------------------*/
-/*    serrano/prgm/project/bigloo/runtime1.3/Clib/ports.c ...          */
+/*    serrano/prgm/project/bigloo/runtime1.9/Clib/ports.c              */
 /*                                                                     */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Thu Jul 23 15:34:53 1992                          */
-/*    Last change :  Sat Jun 12 12:20:02 1993  (serrano)               */
+/*    Last change :  Mon Apr  1 15:27:51 1996 (serrano)                */
 /*                                                                     */
 /*    La manipulation des ports d'entree et de sortie                  */
 /*---------------------------------------------------------------------*/
 #include <stdio.h>
 #include <errno.h>
-#include <termio.h>
-#if( !defined( sony_news ) )
+#if( !(defined( NeXT ) && defined( mc68000 )) )
+#   include <termio.h>
+#endif
+#if( !defined( sony_news ) && !(defined( NeXT ) && defined( mc68000 )) )
 #   include <unistd.h>
 #endif
 #include <sys/file.h>
+/*---------------------------------------------------------------------*/
+/*    On definit cette macros pour que le fichier `gc_private.h' ne    */
+/*    soit pas inclus. Il faut faire ca car ce fichier declare un      */
+/*    prototype de `sbrk' qui n'est pas compatible avec le prototype   */
+/*    quelque fois present (sur linux, par exemple) dans `unisys.h'    */
+/*---------------------------------------------------------------------*/
+#if( defined( i386 ) )
+#   define GC_PRIVATE_H
+#endif
 #include <bigloo.h>
+#if( defined( sony_news ) || (defined( NeXT ) && defined( mc68000 )) )
+#   include <ctype.h>
+#endif
 
+#include <string.h>
 /*---------------------------------------------------------------------*/
 /*    Des variables globales                                           */
 /*---------------------------------------------------------------------*/
 obj_t current_output_port, current_input_port, current_error_port;
-obj_t default_io_bufsiz;
+long default_io_bufsiz;
+
+/*---------------------------------------------------------------------*/
+/*    External definitions.                                            */
+/*---------------------------------------------------------------------*/
+extern obj_t  string_to_bstring();
+extern obj_t  string_to_symbol();
+extern obj_t  make_real();
 
 /*---------------------------------------------------------------------*/
 /*     make_output_port ...                                            */
@@ -66,11 +88,11 @@ FILE *file;
 {
    obj_t new_output_port;
 
-   new_output_port = MAKE_ATOMIC_OBJECT( OUTPUT_PORT_SIZE,
-                                         HEADER_OUTPUT_PORT );
+   new_output_port = GC_MALLOC( OUTPUT_PORT_SIZE );
    
-   new_output_port->output_port_t.file = file;
-   new_output_port->output_port_t.name = name;
+   new_output_port->output_port_t.header = MAKE_HEADER( OUTPUT_PORT_TYPE, 0 );
+   new_output_port->output_port_t.file   = file;
+   new_output_port->output_port_t.name   = name;
    
    return BREF( new_output_port );
 }
@@ -84,10 +106,10 @@ obj_t name;
 {
    FILE *file;
    
-   if( !(file = fopen( STRING( name ).string, "w" )) )
+   if( !(file = fopen( BSTRING_TO_STRING( name ), "w" )) )
       return BFALSE;
 
-   return make_output_port( STRING( name ).string, file );
+   return make_output_port( BSTRING_TO_STRING( name ), file );
 }
 
 /*---------------------------------------------------------------------*/
@@ -99,10 +121,33 @@ obj_t name;
 {
    FILE *file;
    
-   if( !(file = fopen( STRING( name ).string, "w+" )) )
+   if( !(file = fopen( BSTRING_TO_STRING( name ), "a+" )) )
       return BFALSE;
 
-   return make_output_port( STRING( name ).string, file );
+   return make_output_port( BSTRING_TO_STRING( name ), file );
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    open_output_string ...                                           */
+/*---------------------------------------------------------------------*/
+obj_t
+open_output_string()
+{
+   char *buffer;
+   obj_t port;
+
+   port = GC_MALLOC( OUTPUT_STRING_PORT_SIZE );
+
+   /* on alloue le buffer avec des 0 partout */
+   buffer = (char *)(GC_MALLOC( OUTPUT_STRING_PORT_BUFFER_SIZE + 1 ));
+
+   port->output_string_port_t.header = MAKE_HEADER( OUTPUT_STRING_PORT_TYPE,0);
+   port->output_string_port_t.buffer = buffer;
+   port->output_string_port_t.size   = OUTPUT_STRING_PORT_BUFFER_SIZE;
+   port->output_string_port_t.offset = 0;
+   
+   return BREF( port );
 }
 
 /*---------------------------------------------------------------------*/
@@ -112,9 +157,21 @@ obj_t
 close_output_port( port )
 obj_t port;
 {
-   fclose( OUTPUT_PORT( port ).file );
+   if( OUTPUT_STRING_PORTP( port ) )
+   {
+      obj_t res;
+	   
+      res = string_to_bstring( OUTPUT_STRING_PORT( port ).buffer );
+      OUTPUT_PORT( port ).file = 0L;
 
-   return port;
+      return res;
+   }
+   else
+   {
+      fclose( OUTPUT_PORT( port ).file );
+      
+      return port;
+   }
 }
 
 /*---------------------------------------------------------------------*/
@@ -125,42 +182,45 @@ obj_t port;
 /*    de coherence de remplir le buffer.                               */
 /*---------------------------------------------------------------------*/
 obj_t
-open_input_file( name, bufsiz )
-obj_t name, bufsiz;
+open_input_file( name, bbufsiz )
+obj_t name;
+obj_t bbufsiz;
 {
    FILE *file;
    obj_t new_input_port;
-   int   buflen;
-   int   c_bufsiz = CINT( bufsiz );
-   char *buffer;
+   long  buflen;
+   long  bufsiz = CINT( bbufsiz );
    
-   if( !(file = fopen( STRING( name ).string, "r" )) )
+   if( !(file = fopen( BSTRING_TO_STRING( name ), "r" )) )
       return BFALSE;
       
-   new_input_port = MAKE_ATOMIC_OBJECT( INPUT_PORT_SIZE + c_bufsiz + 1,
-                                        HEADER_INPUT_PORT );
+   new_input_port = GC_MALLOC( INPUT_PORT_SIZE + bufsiz + 1 );
 
+   new_input_port->input_port_t.header   = MAKE_HEADER( INPUT_PORT_TYPE, 0 );
    new_input_port->input_port_t.class    = CLASS_FILE;
    new_input_port->input_port_t.file     = file;
-   new_input_port->input_port_t.eof      = 0L;
-   new_input_port->input_port_t.bufsiz   = ADD_I( bufsiz, BINT( 1 ) );
-   new_input_port->input_port_t.backward = BINT( 0 );
-   new_input_port->input_port_t.forward  = BINT( 0 );
-   new_input_port->input_port_t.remember = BINT( 0 );
-   new_input_port->input_port_t.name     = STRING( name ).string;
-   
+   new_input_port->input_port_t.filepos  = 0L;
+   new_input_port->input_port_t.eof      = 0;
+   new_input_port->input_port_t.bufsiz   = bufsiz + 1;
+   new_input_port->input_port_t.backward = 0L;
+   new_input_port->input_port_t.forward  = 0L;
+   new_input_port->input_port_t.remember = 0L;
+   new_input_port->input_port_t.name     = BSTRING_TO_STRING( name );
+   new_input_port->input_port_t.annexe   = 0L;
+   new_input_port->input_port_t.anxsiz   = 0L;
+	
 /*--- On remplis le buffer --------------------------------------------*/
-   buflen = fread( (char *)&(new_input_port->input_port_t.buffer),
+   buflen = fread( BUFFER( BREF( new_input_port ) ),
                    1,
-                   c_bufsiz,
+                   bufsiz,
                    file );
 
 /*--- A-t-on lu la fin du fichier ? -----------------------------------*/
-   if( buflen < c_bufsiz )
-      new_input_port->input_port_t.eof = BINT( 1 );
+   if( buflen < bufsiz )
+      new_input_port->input_port_t.eof = 1;
 
 /*--- On n'oublie pas de placer le `\0' de fin de buffer  -------------*/
-   ((char *)&(new_input_port->input_port_t.buffer))[ buflen ] = '\0';
+   BUFFER( BREF( new_input_port ) )[ buflen ] = '\0';
 
    return BREF( new_input_port );
 }
@@ -172,24 +232,25 @@ obj_t
 open_input_console()
 {
    obj_t new_input_port;
-   int   buflen;
-   int   c_bufsiz = CINT( default_io_bufsiz );
-   char *buffer;
+   long  bufsiz = default_io_bufsiz;
    
-   new_input_port = MAKE_OBJECT( INPUT_PORT_SIZE + c_bufsiz + 1,
-                                 HEADER_INPUT_PORT );
+   new_input_port = GC_MALLOC( INPUT_PORT_SIZE + bufsiz + 1 );
 
+   new_input_port->input_port_t.header   = MAKE_HEADER( INPUT_PORT_TYPE, 0 );
    new_input_port->input_port_t.class    = CLASS_CONSOLE;
    new_input_port->input_port_t.file     = stdin;
-   new_input_port->input_port_t.eof      = 0L;
-   new_input_port->input_port_t.bufsiz   = ADD_I( default_io_bufsiz, BINT( 1 ) );
+   new_input_port->input_port_t.filepos  = 0L;
+   new_input_port->input_port_t.eof      = 0;
+   new_input_port->input_port_t.bufsiz   = default_io_bufsiz + 1;
    new_input_port->input_port_t.backward = default_io_bufsiz;
    new_input_port->input_port_t.forward  = default_io_bufsiz;
    new_input_port->input_port_t.remember = default_io_bufsiz;
    new_input_port->input_port_t.name     = "[stdin]";
+   new_input_port->input_port_t.annexe   = 0L;
+   new_input_port->input_port_t.anxsiz   = 0L;
    
 /*--- On ne remplis pas le buffer, on le vide -------------------------*/
-   bzero( (char *)&(new_input_port->input_port_t.buffer), c_bufsiz );
+   bzero( BUFFER( BREF( new_input_port ) ), bufsiz );
 
 /*--- On vide le buffer de stdin --------------------------------------*/
    fflush( stdin );
@@ -197,6 +258,46 @@ open_input_console()
    return BREF( new_input_port );
 }
 
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    file_to_input_port ...                                           */
+/*---------------------------------------------------------------------*/
+obj_t
+file_to_input_port( file )
+FILE *file;
+{
+   if( file == stdin )
+      open_input_console();
+   else
+   {
+      obj_t new_input_port;
+      long  bufsiz = default_io_bufsiz;
+   
+      new_input_port = GC_MALLOC( INPUT_PORT_SIZE + bufsiz + 1 );
+
+      new_input_port->input_port_t.header   = MAKE_HEADER( INPUT_PORT_TYPE, 0);
+      new_input_port->input_port_t.class    = CLASS_FILE;
+      new_input_port->input_port_t.file     = file;
+      new_input_port->input_port_t.filepos  = 0L;
+      new_input_port->input_port_t.eof      = 0;
+      new_input_port->input_port_t.bufsiz   = default_io_bufsiz + 1;
+      new_input_port->input_port_t.backward = default_io_bufsiz;
+      new_input_port->input_port_t.forward  = default_io_bufsiz;
+      new_input_port->input_port_t.remember = default_io_bufsiz;
+      new_input_port->input_port_t.name     = "[file]";
+      new_input_port->input_port_t.annexe   = 0L;
+      new_input_port->input_port_t.anxsiz   = 0L;
+   
+/*--- On ne remplis pas le buffer, on le vide -------------------------*/
+      bzero( BUFFER( BREF( new_input_port ) ), bufsiz );
+
+/*--- On vide le buffer de file ---------------------------------------*/
+      fflush( file );
+      
+      return BREF( new_input_port );
+   }
+}
+
 /*---------------------------------------------------------------------*/
 /*    open_input_string ...                                            */
 /*---------------------------------------------------------------------*/
@@ -205,28 +306,28 @@ open_input_string( string )
 obj_t string;
 {
    obj_t new_input_port;
-   int   c_bufsiz = CINT( STRING_LENGTH( string ) );
-   char *buffer;
+   long  bufsiz = STRING_LENGTH( string );
 
-   new_input_port = MAKE_OBJECT( INPUT_PORT_SIZE + c_bufsiz + 1,
-                                 HEADER_INPUT_PORT );
+   new_input_port = GC_MALLOC( INPUT_PORT_SIZE + bufsiz + 1 );
 
+   new_input_port->input_port_t.header   = MAKE_HEADER( INPUT_PORT_TYPE, 0 );
    new_input_port->input_port_t.class    = CLASS_STRING;
    new_input_port->input_port_t.file     = 0L;
-   new_input_port->input_port_t.eof      = 0L;
-   new_input_port->input_port_t.bufsiz   = ADD_I( BINT( c_bufsiz ),
-                                                  BINT( 1 ) );
-   new_input_port->input_port_t.backward = BINT( 0 );
-   new_input_port->input_port_t.forward  = BINT( 0 );
-   new_input_port->input_port_t.remember = BINT( 0 );
+   new_input_port->input_port_t.filepos  = 0L;
+   new_input_port->input_port_t.eof      = 0;
+   new_input_port->input_port_t.bufsiz   = bufsiz + 1;
+   new_input_port->input_port_t.backward = 0L;
+   new_input_port->input_port_t.forward  = 0L;
+   new_input_port->input_port_t.remember = 0L;
    new_input_port->input_port_t.name     = "[string]";
-    
+   new_input_port->input_port_t.annexe   = 0L;
+   new_input_port->input_port_t.anxsiz   = 0L;
+	
 /*--- On remplis le buffer --------------------------------------------*/
-   strcpy( (char *)&(new_input_port->input_port_t.buffer),
-           STRING( string ).string );
-
+   strcpy( BUFFER( BREF( new_input_port ) ), BSTRING_TO_STRING( string ) );
+	
 /*--- On a lu la fin du fichier (bien sur) ! --------------------------*/
-   new_input_port->input_port_t.eof = BINT( 1 );
+   new_input_port->input_port_t.eof = 1;
 
    return BREF( new_input_port );   
 }
@@ -241,19 +342,19 @@ obj_t s , f;
    FILE *file = OUTPUT_PORT( f ).file;
    
    fprintf( file, "====== input_port_debug ===============================\n" );
-   fprintf( file, "\n" );
+   fprintf( file, "%s:\n\n", INPUT_PORT( s ).name );
    fprintf( file, "   backward   : %d '%c'\n",
-            CINT( INPUT_PORT( s ).backward ),
-            BUFFER( s )[ CINT( INPUT_PORT( s ).backward ) ]);
+            INPUT_PORT( s ).backward,
+            BUFFER( s )[ INPUT_PORT( s ).backward ]);
    fprintf( file, "   forward    : %d '%c'\n",
-            CINT( INPUT_PORT( s ).forward ),
-            BUFFER( s )[ CINT( INPUT_PORT( s ).forward ) ]);
+            INPUT_PORT( s ).forward,
+            BUFFER( s )[ INPUT_PORT( s ).forward ]);
    fprintf( file, "   remember   : %d '%c'\n",
-            CINT( INPUT_PORT( s ).remember ),
-            BUFFER( s )[ CINT( INPUT_PORT( s ).remember ) ]);
+            INPUT_PORT( s ).remember,
+            BUFFER( s )[ INPUT_PORT( s ).remember ]);
    fprintf( file, "   mark       : %d '%c'\n",
-            CINT( INPUT_PORT( s ).mark ),
-            BUFFER( s )[ CINT( INPUT_PORT( s ).mark ) ]);
+            INPUT_PORT( s ).mark,
+            BUFFER( s )[ INPUT_PORT( s ).mark ]);
    fprintf( file, "   eof        : %d\n", INPUT_PORT( s ).eof );
    fprintf( file, "   buffer     : [%s]\n", BUFFER( s ) );
    fprintf( file, "===================================================\n" );
@@ -269,15 +370,60 @@ close_input_port( port )
 obj_t port;
 {
    if( (INPUT_PORTP( port )) && (INPUT_PORT( port ).class == CLASS_FILE) )
-   {
       fclose( INPUT_PORT( port ).file );
 
-      strcpy( BUFFER( port ), "input port closed" );
-   }
-
    return port;
 }
 
+/*---------------------------------------------------------------------*/
+/*    void                                                             */
+/*    buffer_to_annexe ...                                             */
+/*    -------------------------------------------------------------    */
+/*    On deplace le buffer dans l'annexe. Deux cas peuvent se          */
+/*    presenter:                                                       */
+/*       - l'annexe n'existe pas, on la creer de la taille du          */
+/*         buffer                                                      */
+/*       - l'annexe existe, on la fait grandir.                        */
+/*---------------------------------------------------------------------*/
+void
+buffer_to_annexe( port )
+obj_t port;
+{
+   if( !INPUT_PORT( port ).annexe )
+   /* on est dans le premier cas, l'annexe n'existe pas */
+   {
+      char *annexe;
+      long  size = INPUT_PORT( port ).bufsiz - 1;
+      
+      annexe = malloc( size );
+      
+      memcpy( annexe, BUFFER( port ), size );
+      
+      INPUT_PORT( port ).anxsiz = size;
+      INPUT_PORT( port ).annexe = annexe;
+   }
+   else
+   /* on est dans le deuxieme cas, l'annexe existe deja */
+   {
+      char *annexe;
+      long  size = INPUT_PORT( port ).bufsiz + INPUT_PORT( port ).anxsiz - 1 ;
+      
+      free( INPUT_PORT( port ).annexe );
+      
+      annexe = malloc( size );
+      
+      memcpy( annexe,
+	      INPUT_PORT( port ).annexe,
+	      INPUT_PORT( port ).anxsiz );
+      memcpy( annexe + INPUT_PORT( port ).anxsiz,
+	      BUFFER( port ),
+	      INPUT_PORT( port ).bufsiz );
+      
+      INPUT_PORT( port ).anxsiz = size;
+      INPUT_PORT( port ).annexe = annexe;
+   }
+}
+
 /*---------------------------------------------------------------------*/
 /*    input_port_fill_buffer ...                                       */
 /*    -------------------------------------------------------------    */
@@ -308,33 +454,63 @@ obj_t port;
 /*       +-+-----------------+-+---------------------------------+     */
 /*     0 = backward        forward                          bufsiz     */
 /*---------------------------------------------------------------------*/
-unsigned char
+bool_t
 input_port_fill_buffer( port )
 obj_t port;
 {
-   int backward = CINT( INPUT_PORT( port ).backward );
+   long backward = INPUT_PORT( port ).backward;
 
 /*--- du debug --------------------------------------------------------*/
 #ifdef DEBUG_PORT
    puts( "Avant le fill_buffer: " );
    input_port_debug( port, current_output_port );
-#endif      
+#endif
+	
+/*--- le buffer est plein, on deplace les car lus dans l'annexe -------*/
+   if( !INPUT_PORT( port ).eof && !backward )
+   {
+      if( (INPUT_PORT( port ).class != CLASS_CONSOLE) )
+      {
+#ifdef DEBUG_PORT
+	 puts( "Le buffer est plein" );
+#endif 		
+	 /* on copie le buffer dans l'annexe */
+	 buffer_to_annexe( port );
+
+	 /* on met les marks dans un etat coherent */
+	 backward = INPUT_PORT( port ).bufsiz - 1;
+	 INPUT_PORT( port ).backward = backward;
+      }
+      else
+      {
+	 if( INPUT_PORT( port ).forward == INPUT_PORT( port ).bufsiz )
+	 {
+	    the_failure( string_to_bstring( "input_port_fill_buffer" ),
+		         string_to_bstring( "token too large " ),
+		         string_to_bstring( "on console" ) );
+	    exit( -1 );
+	 }
+      }
+   }
+
 /*---------------------------------------------------------------------*/
 /*    On a un traitement specifique a faire suivant qu'on est en       */
 /*    train de lire sur la console ou dans un fichier                  */
 /*---------------------------------------------------------------------*/
    if( INPUT_PORT( port ).class == CLASS_FILE )
    {
-      if( INPUT_PORT( port ).eof || !backward )
+/*--- On a lu un `fin de fichier', il faut donc s'arretter ------------*/
+      if( INPUT_PORT( port ).eof )
          return 0;
       else
       {
-         char *buffer = BUFFER( port );
-         int   bufsiz = CINT( INPUT_PORT( port ).bufsiz );
-         int   buflen;
+         char *buffer  = BUFFER( port );
+         long  bufsiz  = INPUT_PORT( port ).bufsiz;
+	 long  forward = INPUT_PORT( port ).forward;
+         long  buflen;
 
 /*--- On recale le buffer a gauche ------------------------------------*/
-         strcpy( buffer, buffer + backward );
+	 strcpy( buffer, buffer + backward );
 
 /*--- Ou commence et ou termine le buffer actuel ?  -------------------*/
          buflen = bufsiz - backward - 1;
@@ -342,10 +518,10 @@ obj_t port;
 /*--- On lit pour remplir le buffer -----------------------------------*/
          buflen += fread( buffer + buflen, 1, bufsiz - buflen - 1,
                           INPUT_PORT( port ).file );
-
+			
 /*--- A-t-on lu la fin du fichier ? -----------------------------------*/
          if( buflen < (bufsiz - 1) )
-            INPUT_PORT( port ).eof = BINT( 1 );
+            INPUT_PORT( port ).eof = 1;
 
 /*--- On n'oublie pas de placer le `\0' de fin de buffer  -------------*/
          buffer[ buflen ] = '\0';
@@ -354,39 +530,47 @@ obj_t port;
    else
       if( INPUT_PORT( port ).class == CLASS_CONSOLE )
       {
-         char *buffer = BUFFER( port );
-         int   bufsiz = CINT( INPUT_PORT( port ).bufsiz );
-         int   buflen;
+         char *buffer  = BUFFER( port );
+         long  bufsiz  = INPUT_PORT( port ).bufsiz;
+	 long  forward = INPUT_PORT( port ).forward;
+         long  buflen;
 
-         if( INPUT_PORT( port ).eof ||
-             (CINT( INPUT_PORT( port ).forward ) == (bufsiz - 1)) )
-            return 0;
+         if( INPUT_PORT( port ).eof )
+	    return 0;
          else
          {
 /*--- On recale le buffer a gauche ------------------------------------*/
             strcpy( buffer, buffer + backward );
             
-            buflen = CINT( INPUT_PORT( port ).forward )-backward - 1;
+            buflen = forward - backward - 1;
             
             fgets( buffer + buflen,
                    bufsiz - buflen - 1,
-                   INPUT_PORT( current_input_port ).file );
+                   INPUT_PORT( port ).file );
             
-            if( feof( INPUT_PORT( current_input_port ).file ) )
-            INPUT_PORT( port ).eof = BINT( 1 );
+            if( feof( INPUT_PORT( port ).file ) )
+	    {
+	       INPUT_PORT( port ).eof = 1;
+
+               if( buffer[ buflen ] == '\0' )
+	       {
+		  INPUT_PORT( port ).remember = buflen;
+		  
+		  return 0;
+	       }
+	    }
          }
       }
       else
          return 0;
          
 /*--- Et ziou, c'est fini... ------------------------------------------*/
-   INPUT_PORT( port ).forward  = SUB_I_PTAG( SUB_I( INPUT_PORT( port ).forward,
-                                             INPUT_PORT( port ).backward ),
-                                             PSUB_TAG( 1 ) );
-   INPUT_PORT( port ).remember = SUB_I( INPUT_PORT( port ).remember,
-                                      INPUT_PORT( port ).backward );
+   INPUT_PORT( port ).forward  = INPUT_PORT( port ).forward -
+                                 INPUT_PORT( port ).backward - 1;
+   INPUT_PORT( port ).remember = INPUT_PORT( port ).remember -
+                                 INPUT_PORT( port ).backward;
       
-   INPUT_PORT( port ).backward = BINT( 0 );
+   INPUT_PORT( port ).backward = 0L;
       
 /*--- du debug --------------------------------------------------------*/
 #ifdef DEBUG_PORT
@@ -397,6 +581,32 @@ obj_t port;
    return 1;
 }
 
+/*---------------------------------------------------------------------*/
+/*    obj                                                              */
+/*    reset_console ...                                                */
+/*    -------------------------------------------------------------    */
+/*    On purge les ports d'entree pour que le ^C marche bien sous      */
+/*    l'interprete. Le seul entroit ou cette fonction est utilisee     */
+/*    la fonction `repl' (voir Eval/eval.scm).                         */
+/*---------------------------------------------------------------------*/
+obj_t
+reset_console( port )
+obj_t port;
+{
+   long size = INPUT_PORT( port ).bufsiz - 1;
+
+   if( INPUT_PORT( port ).class == CLASS_CONSOLE )
+   {
+      INPUT_PORT( port ).backward = size;
+      INPUT_PORT( port ).forward  = size;
+      INPUT_PORT( port ).remember = size;
+      bzero( BUFFER( port ) , size );
+      fflush( stdin );
+   }
+
+   return BUNSPEC;
+}
+
 /*---------------------------------------------------------------------*/
 /*    input_port_get_string ...                                        */
 /*    -------------------------------------------------------------    */
@@ -412,11 +622,37 @@ obj_t s;
    char  bck;
    obj_t res;
 
-   bck = BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) ];
+   bck = BUFFER( s )[ INPUT_PORT( s ).backward ];
    
-   BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) ] = '\0';
-   res = c_string_to_string( &BUFFER( s )[ CINT( INPUT_PORT( s ).mark ) ] );
-   BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) ] = bck;
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = '\0';
+
+/* printf( "get_small_string:  mark: %d\n", INPUT_PORT( s ).mark );    */
+/* printf( "                 annexe: %s\n", INPUT_PORT( s ).annexe );  */
+/* printf( "                 buffer: %s\n", BUFFER( s ) );  */
+	
+   if( INPUT_PORT( s ).annexe )
+   {
+      char *aux;
+
+      aux = malloc( INPUT_PORT_GET_LENGTH( s ) );
+
+      memcpy( aux, INPUT_PORT( s ).annexe, INPUT_PORT( s ).anxsiz );
+      memcpy( &(aux[ INPUT_PORT( s ).anxsiz ]),
+	      BUFFER( s ),
+	      INPUT_PORT( s ).backward );
+				       
+      res = string_to_bstring( aux );
+
+      free( aux );
+      free( INPUT_PORT( s ).annexe );
+      
+      INPUT_PORT( s ).anxsiz = 0L;
+      INPUT_PORT( s ).annexe = 0L;
+   }
+   else
+      res = string_to_bstring( &BUFFER( s )[ INPUT_PORT( s ).mark ] );
+	
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = bck;
 
    return res;
 }
@@ -437,16 +673,44 @@ obj_t s;
 /*---------------------------------------------------------------------*/
 obj_t
 input_port_get_small_string( s )
-obj_t s;
+obj_t s; 
 {
    char  bck;
    obj_t res;
    
-   bck = BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) - 1 ];
+   bck = BUFFER( s )[ INPUT_PORT( s ).backward - 1 ];
    
-   BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) - 1 ] = '\0';
-   res = c_string_to_string( &BUFFER( s )[ CINT( INPUT_PORT( s ).mark ) + 1 ] );
-   BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) - 1 ] = bck;
+   BUFFER( s )[ INPUT_PORT( s ).backward - 1 ] = '\0';
+
+/* printf( "get_small_string:  mark: %d\n", ( INPUT_PORT( s ).mark ) );*/
+/* printf( "                 annexe: %s\n", INPUT_PORT( s ).annexe );  */
+/* printf( "                 buffer: %s\n", BUFFER( s ) );  */
+	
+   if( INPUT_PORT( s ).annexe )
+   {
+      char *aux;
+
+      aux = malloc( INPUT_PORT_GET_LENGTH( s ) );
+      
+      memcpy( aux,
+	      &INPUT_PORT( s ).annexe[ 1 ],
+	      INPUT_PORT( s ).anxsiz - 1 );
+      memcpy( &(aux[ INPUT_PORT( s ).anxsiz - 1 ]),
+	      BUFFER( s ),
+	      INPUT_PORT( s ).backward - 1 );
+				       
+      res = string_to_bstring( aux );
+
+      free( INPUT_PORT( s ).annexe );
+      free( aux );
+      
+      INPUT_PORT( s ).anxsiz = 0L;
+      INPUT_PORT( s ).annexe = 0L;
+   }
+   else
+      res = string_to_bstring(&BUFFER( s )[ INPUT_PORT( s ).mark + 1 ]);
+	
+   BUFFER( s )[ INPUT_PORT( s ).backward - 1 ] = bck;
 
    return res;
 }
@@ -460,12 +724,106 @@ obj_t s;
 {
    char  bck;
    obj_t res;
+   char  *buf;
+	
+   bck = BUFFER( s )[ INPUT_PORT( s ).backward ];
    
-   bck = BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) ];
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = '\0';
+
+   if( INPUT_PORT( s ).annexe )
+   {
+      buf = malloc( INPUT_PORT_GET_LENGTH( s ) );
+      
+      memcpy( buf, INPUT_PORT( s ).annexe, INPUT_PORT( s ).anxsiz );
+      memcpy( &(buf[ INPUT_PORT( s ).anxsiz ]),
+	      BUFFER( s ),
+	      INPUT_PORT( s ).backward );
+   }
+   else
+      buf = &BUFFER( s )[ INPUT_PORT( s ).mark ];
+		
+/*---------------------------------------------------------------------*/
+/*    Veut-on des symbols `case unsensitive' ?                         */
+/*---------------------------------------------------------------------*/
+#define SYMBOL_UNCASE_SENSITIVE 1
+
+#if defined( SYMBOL_UNCASE_SENSITIVE )
+   {
+      char *walk;
+
+      for( walk = buf; *walk; walk++ )
+         if( isascii( *walk ) )
+            *walk = toupper( *walk );
+   }
+#endif   
+
+   res = string_to_symbol( buf );
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = bck;
+
+   if( INPUT_PORT( s ).annexe )
+   {
+      free( buf );
+      free( INPUT_PORT( s ).annexe );
+		
+      INPUT_PORT( s ).anxsiz = 0L;
+      INPUT_PORT( s ).annexe = 0L;
+   }
+		
+   return res;
+}
+
+/*---------------------------------------------------------------------*/
+/*    input_port_get_fixnum ...                                        */
+/*---------------------------------------------------------------------*/
+obj_t
+input_port_get_fixnum( s )
+obj_t s;
+{
+   char bck;
+   long res;
+
+   bck = BUFFER( s )[ INPUT_PORT( s ).backward ];
    
-   BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) ] = '\0';
-   res = c_string_to_symbol( &BUFFER( s )[ CINT( INPUT_PORT( s ).mark ) ] );
-   BUFFER( s )[ (int)CINT( INPUT_PORT( s ).backward ) ] = bck;
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = '\0';
+
+   if( INPUT_PORT( s ).annexe )
+   {
+      FAILURE( string_to_bstring( "get-flonum" ),
+	       string_to_bstring( "Token too large" ),
+	       BUNSPEC );
+   }
+   else
+      res = atol( &BUFFER( s )[ INPUT_PORT( s ).mark ] );
+	
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = bck;
+
+   return BINT( res );
+}
+
+/*---------------------------------------------------------------------*/
+/*    input_port_get_flonum ...                                        */
+/*---------------------------------------------------------------------*/
+obj_t
+input_port_get_flonum( s )
+obj_t s;
+{
+   char  bck;
+   obj_t res;
+
+   bck = BUFFER( s )[ INPUT_PORT( s ).backward ];
+   
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = '\0';
+
+   if( INPUT_PORT( s ).annexe )
+   {
+      FAILURE( string_to_bstring( "get-flonum" ),
+	       string_to_bstring( "Token too large" ),
+	       BUNSPEC );
+   }
+   else
+      res = make_real( strtod( &BUFFER( s )[ INPUT_PORT( s ).mark ], 0 ) );
+   
+   BUFFER( s )[ INPUT_PORT( s ).backward ] = bck;
 
    return res;
 }
@@ -473,74 +831,72 @@ obj_t s;
 /*---------------------------------------------------------------------*/
 /*    intput_port_read_string ...                                      */
 /*    -------------------------------------------------------------    */
+/*    Cette fonction retourne une chaine fraichement allouee           */
+/*    -------------------------------------------------------------    */
 /*    Cette fonction fonctionne aussi bien pour les `consoles' que pour*/
 /*    les fichiers.                                                    */
 /*---------------------------------------------------------------------*/
 obj_t
 intput_port_read_string( port, number )
-obj_t port, number;
+obj_t port;
+long  number;
 {
    obj_t res;
-   int   len;
+   long  len;
    char  aux;
    char *buffer = BUFFER( port );
 
    /* On verifie que la taille de ce qu'on veut lire n'est pas plus grande */
    /* que la taille des buffers du port.                                   */
-   len = GT_I( INPUT_PORT( port ).bufsiz, number ) ?
-      CINT( number ) : CINT( INPUT_PORT( port ).bufsiz );
+   len = INPUT_PORT( port ).bufsiz > number ?
+             number : INPUT_PORT( port ).bufsiz;
 
    /* On regarde si on n'a pas deja lu eof sur le port */
    if( INPUT_PORT( port ).eof )
    {
-      int buf_len;
-      int offset;
+      long buf_len;
+      long offset;
 
       /* Puisqu'on a lu eof, il faut regarder quelle est la taille de la */
       /* chaine contenue dans le buffer.                                 */
-      buf_len = strlen( &BUFFER( port )[ CINT( INPUT_PORT( port ).backward ) ] );
+      buf_len = (long)strlen( &BUFFER( port )[ INPUT_PORT( port ).backward ] );
 
       len = buf_len > len ? len : buf_len;
 
       /* On peut maintenant construire la nouvelle chaine. */
-      offset = CINT( ADD_I( INPUT_PORT( port ).backward, BINT( len ) ) );
+      offset = INPUT_PORT( port ).backward + len;
       
-      aux = buffer[ offset ];
+      aux = buffer[ offset ]; 
       buffer[ offset ] = '\0';
-      res = c_string_to_string( &BUFFER( port )[ CINT( INPUT_PORT( port ).backward ) ] );
+      res = string_to_bstring( &BUFFER( port )[ INPUT_PORT( port ).backward ] );
       buffer[ offset ] = aux;
       
       /* on ajuste les curseurs */
-      INPUT_PORT( port ).forward = ADD_I( INPUT_PORT( port ).backward,
-                                          BINT( len ) );
+      INPUT_PORT( port ).forward = INPUT_PORT( port ).backward + len;
    }
    else
       /* on regarde si on a assez de chose dans le buffer */
-      if( GT_I( SUB_I( INPUT_PORT( port ).bufsiz,
-                       INPUT_PORT( port ).backward ),
-                       number ) )
+      if( (INPUT_PORT( port ).bufsiz - INPUT_PORT( port ).backward) < number )
       {
-         int offset;
+         long offset;
 
          /* on calcule la chaine resultat */
-         offset = CINT( ADD_I( INPUT_PORT( port ).backward, number ) );
+         offset = INPUT_PORT( port ).backward + number;
          
          aux = buffer[ offset ];
          buffer[ offset ] = '\0';
-         res = c_string_to_string( &BUFFER( port )[ CINT( INPUT_PORT( port ).backward ) ] );
+         res = string_to_bstring( &BUFFER( port )[ INPUT_PORT( port ).backward ] );
          buffer[ offset ] = aux;
       
          /* on ajuste les curseurs */
-         INPUT_PORT( port ).forward = ADD_I( INPUT_PORT( port ).backward,
-                                             number );
+         INPUT_PORT( port ).forward = INPUT_PORT( port ).backward + number;
       }
       else
       {
-         int len_aux;
+         long len_aux;
          
          /* on commence par remplir le buffer */
-         INPUT_PORT( port ).forward = ADD_I_PTAG( INPUT_PORT( port ).forward,
-                                                  PSUB_TAG( 1 ) );
+         INPUT_PORT( port ).forward = INPUT_PORT( port ).forward + 1;
          input_port_fill_buffer( port );
 
          /* On regarde la taille de ce qu'on vient de lire */
@@ -549,20 +905,18 @@ obj_t port, number;
             /* on construit la nouvelle chaine */
             aux = buffer[ len ];
             buffer[ len ] = '\0';
-            res = c_string_to_string( BUFFER( port ) );
+            res = string_to_bstring( BUFFER( port ) );
             buffer[ len ] = aux;
       
             /* on ajuste les curseurs */
-            INPUT_PORT( port ).forward = ADD_I( INPUT_PORT( port ).backward,
-                                                BINT( len ) );
+            INPUT_PORT( port ).forward = INPUT_PORT( port ).backward + len;
          }
          else
          {
-            res = c_string_to_string( BUFFER( port ) );
+            res = string_to_bstring( BUFFER( port ) );
       
             /* on ajuste les curseurs */
-            INPUT_PORT( port ).forward = ADD_I( INPUT_PORT( port ).backward,
-                                                BINT( len_aux ) );
+            INPUT_PORT( port ).forward = INPUT_PORT( port ).backward + len_aux;
          }   
       }
 
@@ -580,14 +934,14 @@ input_port_display_error( ip, op )
 obj_t ip, op;
 {
    FILE *output        = OUTPUT_PORT( op ).file;
-   int  forward        = CINT( INPUT_PORT( ip ).forward ) - 1;
-   int  end_of_print   = forward;
-   int  start_of_print = (int)CINT( INPUT_PORT( ip ).backward ) - 40;
-   int  count;
+   long forward        = INPUT_PORT( ip ).forward - 1;
+   long end_of_print   = forward;
+   long start_of_print = INPUT_PORT( ip ).backward - 40;
+   long count;
    char bck;
    
    /* on calcule l'endroit ou on va commencer a afficher */
-   if( (int)start_of_print < 0 )
+   if( (long)start_of_print < 0 )
       start_of_print = 0;
 
    count = end_of_print - start_of_print;
@@ -602,9 +956,8 @@ obj_t ip, op;
    bck = BUFFER( ip )[ end_of_print ];
    BUFFER( ip )[ end_of_print ] = '\0';
 
-   /* on affiche la chaine */
-   fprintf( output, "%s\n",
-            &BUFFER( ip )[ start_of_print ] );
+   /* on affiche la chaine */ 
+   fprintf( output, "%s\n", &BUFFER( ip )[ start_of_print ] );
    
    /* on restore le buffer */
    BUFFER( ip )[ end_of_print ] = bck;
@@ -621,7 +974,7 @@ init_io()
 #   define _SBFSIZ 8
 #endif
 
-   default_io_bufsiz = BINT( BUFSIZ * _SBFSIZ );
+   default_io_bufsiz = BUFSIZ * _SBFSIZ;
    
    current_output_port = make_output_port( "stdout", stdout );
    current_error_port  = make_output_port( "stderr", stderr );
@@ -631,10 +984,96 @@ init_io()
 /*---------------------------------------------------------------------*/
 /*    fexists ...                                                      */
 /*---------------------------------------------------------------------*/
-unsigned char
+bool_t
 fexists( name )
 char *name;
 {
    return !access( name, R_OK );
 }
 
+/*---------------------------------------------------------------------*/
+/*    bool_t                                                           */
+/*    reset_eof ...                                                    */
+/*    -------------------------------------------------------------    */
+/*    Quand le port pointe sur la console, cette fonction annule la    */
+/*    lecture du `^D'. Cela permet une reprise de lecture.             */
+/*---------------------------------------------------------------------*/
+bool_t
+reset_eof( port )
+obj_t port;
+{
+   if( INPUT_PORT( port ).class == CLASS_CONSOLE )
+   {
+      long bufsiz = INPUT_PORT( port ).bufsiz;
+      
+      /* on annule le `eof' */
+      INPUT_PORT( port ).eof = 0;
+      
+      /* on nettoie le buffer */
+      bzero( (char *)BUFFER( port ), bufsiz );
+
+      /* on recale les curseurs */
+      INPUT_PORT( port ).forward  = default_io_bufsiz;
+      INPUT_PORT( port ).remember = default_io_bufsiz;
+      INPUT_PORT( port ).backward = default_io_bufsiz;
+
+      /* on flush , et on clear */
+      fflush( stdin );
+      clearerr( stdin );
+
+      return 1;
+   }
+   else
+      return 0;
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    strport_flush ...                                                */
+/*    -------------------------------------------------------------    */
+/*    On flush un string port.                                         */
+/*---------------------------------------------------------------------*/
+obj_t
+strport_flush( port )
+obj_t port;
+{
+   obj_t res;
+
+   res = string_to_bstring( OUTPUT_STRING_PORT( port ).buffer );
+
+   OUTPUT_STRING_PORT( port ).offset = 0;
+   OUTPUT_STRING_PORT( port ).buffer[ 0 ] = '\0';
+
+   return res;
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    strport_grow ...                                                 */
+/*---------------------------------------------------------------------*/
+obj_t
+strport_grow( 	p )
+obj_t p;
+{
+   long  old_size, new_size;
+   char *old_buffer, *new_buffer;
+	
+   old_buffer = OUTPUT_STRING_PORT( p ).buffer;
+   old_size   = OUTPUT_STRING_PORT( p ).size;
+
+   new_size   = old_size * 2;
+   new_buffer = (char *)( GC_MALLOC( new_size + 1 ) );
+
+   strcpy( new_buffer, old_buffer );
+
+   OUTPUT_STRING_PORT( p ).buffer = new_buffer;
+   OUTPUT_STRING_PORT( p ).size   = new_size;
+
+   return p;
+}
+
+
+
+
+
+
